<?php
/**
 * Created by PhpStorm.
 * User: jderosa
 * Date: 20/04/20
 * Time: 11.57
 */

namespace FiloBlu\Rma\Block\Returns;

use Exception;
use FiloBlu\Rma\Helper\EasyReturnHelper;
use Magento\Eav\Model\Config;
use Magento\Eav\Model\Form\Factory;
use Magento\Framework\App\ProductMetadataInterface;
use Magento\Framework\Data\Collection\ModelFactory;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Framework\Registry;
use Magento\Framework\View\Element\Template\Context;
use Magento\Rma\Helper\Data;
use Magento\Rma\Model\Item\FormFactory;
use Magento\Rma\Model\ItemFactory;
use Magento\Sales\Api\Data\OrderInterface;
use Magento\Sales\Model\Order\Address\Renderer;
use Magento\Store\Model\ScopeInterface;
use Throwable;

/**
 *
 */
class Create extends \Magento\Rma\Block\Returns\Create
{
    const CREATE_TEMPLATE_MAGENTO_V2_3_X = 'FiloBlu_Rma::return/createv23x.phtml';
    const CREATE_TEMPLATE_MAGENTO_V2_2_X = 'FiloBlu_Rma::return/create.phtml';
    const CREATE_TEMPATE_MAGENTO_ORIGINAL = 'Magento_Rma::return/create.phtml';

    /**
     * @var ProductMetadataInterface
     */
    private $productMetadata;

    /**
     * @var EasyReturnHelper
     */
    private $easyReturnHelper;

    /**
     * @var Context
     */
    private $context;

    /**
     * Create constructor.
     * @param Context $context
     * @param ModelFactory $modelFactory
     * @param Factory $formFactory
     * @param Config $eavConfig
     * @param ItemFactory $itemFactory
     * @param FormFactory $itemFormFactory
     * @param Data $rmaData
     * @param Registry $registry
     * @param Renderer $addressRenderer
     * @param ProductMetadataInterface $productMetadata
     * @param EasyReturnHelper $easyReturnHelper
     * @param array $data
     */
    public function __construct(
        Context                  $context,
        ModelFactory             $modelFactory,
        Factory                  $formFactory,
        Config                   $eavConfig,
        ItemFactory              $itemFactory,
        FormFactory              $itemFormFactory,
        Data                     $rmaData,
        Registry                 $registry,
        Renderer                 $addressRenderer,
        ProductMetadataInterface $productMetadata,
        EasyReturnHelper         $easyReturnHelper,
        array                    $data = [])
    {
        parent::__construct($context,
            $modelFactory,
            $formFactory,
            $eavConfig,
            $itemFactory,
            $itemFormFactory,
            $rmaData,
            $registry,
            $addressRenderer,
            $data);
        $this->productMetadata = $productMetadata;
        $this->easyReturnHelper = $easyReturnHelper;
        $this->context = $context;
    }

    /**
     * @param string $template
     * @return \Magento\Rma\Block\Returns\Create
     */
    public function setTemplate($template)
    {
        if (!preg_match('/return\/create/', $template)) {
            return parent::setTemplate($template);
        }

        try {
            if (!$this->easyReturnHelper->isEasyReturnEnabled($this->context->getStoreManager()->getStore()->getId())) {
                return parent::setTemplate(self::CREATE_TEMPATE_MAGENTO_ORIGINAL);
            }

            $magentoVersion = $this->productMetadata->getVersion();

            if (version_compare($magentoVersion, '2.3.0') != -1) {
                $template = self::CREATE_TEMPLATE_MAGENTO_V2_3_X;
            } else {
                $template = self::CREATE_TEMPLATE_MAGENTO_V2_2_X;
            }

            return parent::setTemplate($template);
        } catch (Exception $exception) {
            $this->_logger->critical($exception->getMessage(), ['exception' => $exception]);
            return parent::setTemplate(self::CREATE_TEMPATE_MAGENTO_ORIGINAL);
        } catch (Throwable $throwable) {
            $this->_logger->critical($throwable->getMessage(), ['exception' => $throwable]);
            return parent::setTemplate(self::CREATE_TEMPATE_MAGENTO_ORIGINAL);
        }
    }

    /**
     * @param OrderInterface $order
     * @return bool
     * @throws NoSuchEntityException
     */
    public function isRegionRequired(OrderInterface $order)
    {
        $storeId = $this->context->getStoreManager()->getStore()->getId();
        $country = $this->_scopeConfig->getValue('general/region/state_required', ScopeInterface::SCOPE_STORE, $storeId) ?? '';
        $myCountry = $order->getShippingAddress()->getCountryId();

        if ($country != '') {
            if (in_array($myCountry, explode(',', $country))) {
                return true;
            }
        }
        return false;
    }


    /**
     * @return mixed
     */
    public function getItems()
    {
        // Temporary workaround PEUT-1618
        $items = $this->getData('items');
        $returnableItems = [];
        foreach ($items as $item) {
            if ($item->getPrice() > 0) {
                $returnableItems [] = $item;
            }
        }

        return $returnableItems;
    }
}
